#!/bin/sh

# Get the directory where this script resides.
WORK_DIR=$(cd `dirname $0` && pwd) 
cd "${WORK_DIR}"

# If the tests need to invoke LSB C or C++ compiler, pkg-config data path
# will also be set accordingly.
# $CC and/or $CXX must be set appropriately by the caller in this case.
USE_LSB_COMPILER=""
echo "x${CC}" | grep "lsbc" > /dev/null
if [ $? -eq 0 ]; then
    USE_LSB_COMPILER=yes
else
    echo "x${CXX}" | grep "lsbc" > /dev/null
    if [ $? -eq 0 ]; then
        USE_LSB_COMPILER=yes
    fi
fi

if [ "t${USE_LSB_COMPILER}" = "tyes" ]; then
    for x in /opt/lsb/lib*/pkgconfig; do
        PKG_CONFIG_PATH="${x}${PKG_CONFIG_PATH+:${PKG_CONFIG_PATH}}"
    done
    export PKG_CONFIG_PATH
fi

#remove previous results and temporaries
rm -rf results
mkdir results

JOURNAL="${WORK_DIR}/results/test.journal"

printf "Test execution started at `date` \n" >> "${JOURNAL}"

# run the tests
SUITE_PATTERN="\-t2c"
if test -n "$1"; then
    SUITE_PATTERN="$1"
fi

for SUITE in *-t2c; do
    cd "${WORK_DIR}"
	
    printf "${SUITE}" | grep "${SUITE_PATTERN}" > /dev/null
    if test $? -eq 0 -a -d "${SUITE}/tests/"; then
        cd "${SUITE}/tests/"
        
        # for each subdirectory containing an executable with the same name...
        for TEST_NAME in *; do
            if test -f "${TEST_NAME}/${TEST_NAME}"; then
                LOG_FILE="${WORK_DIR}/${SUITE}/tests/${TEST_NAME}/${TEST_NAME}.log"
                
                rm -rf "${LOG_FILE}"
                cd "${TEST_NAME}" && ./${TEST_NAME} && cd ..
                
                if test -f "${LOG_FILE}"; then
                    cat "${LOG_FILE}" >> "${JOURNAL}"
                fi
            fi
            
            cd "${WORK_DIR}/${SUITE}/tests/"
        done
    fi
done

cd "${WORK_DIR}"
printf "Test execution completed at `date` \n" >> "${JOURNAL}"
    
exit 0
