/**************************************************************************
Copyright (C) <$YEAR$> <$COPYRIGHT_HOLDER$>. All rights reserved.

<$LICENSE$>
**************************************************************************/

///////////////////////////////////////////////////////////////////////////
// This file contains tests for the following library: 
//     <$library$>, 
// section: 
//     "<$libsection$>"
// 
// Test count: <$num_tests$>
//
// This file was created using T2C ("Template-to-Code") system:
// http://sourceforge.net/projects/template2code/
///////////////////////////////////////////////////////////////////////////

#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <assert.h>

#include <tet_api.h>
#include <t2c_api.h>

// 'suite_root_dir_' will be filled with an absolute path corresponding to
// 'suite_root_dir_rel_' in startup_func(). This absolute path will be used
// by T2C_GET_DATA_PATH to produce absolute paths to the specified files, etc.
//
// The test must be executed "in place", so that the relative path to the 
// suite root was fixed.
const char* suite_root_dir_rel_ = "../..";
char suite_root_dir_[PATH_MAX + 1];

const char* test_name_ = "<$T2C_FILE_NAME$>";
const int num_tests_ = <$num_tests$>;
int tests_passed_ = 0;

unsigned int wait_time_ = 30;
int single_process_ = 0;     

#ifdef CHECK_EXT_REQS
    int check_ext_ = 1;
#else
    int check_ext_ = 0;
#endif

int init_failed_ = FALSE;
char* init_fail_reason_ = NULL;

///////////////////////////////////////////////////////////////////////////
// The directory where the files containing requirements are located:
const char* rcat_dir_ = "../../reqs";

// Catalogue(s) of requirements to be used (empty strings are ignored):
const char* rcat_names_[] = {
    "<$T2C_FILE_NAME$>",
    <$if concat(req_cat)$>"<$req_cat : join(",\n    ")$>",
    <$endif$>NULL
};

// Full catalogue of requirements to be used in the tests.
CReqCat* rcat_ = NULL;

///////////////////////////////////////////////////////////////////////////
// Utility functions - declarations
static void 
test_launcher();

static void 
startup_func();

static void 
cleanup_func();

static int
special_startup_func();

///////////////////////////////////////////////////////////////////////////
// User-defined globals 
<$global$>

///////////////////////////////////////////////////////////////////////////
// Startup and cleanup functions

static void 
startup_func()
{
    const char* reason_to_cancel_ = NULL;
    init_failed_ = FALSE;
    
    // initialize common settings: wait time, etc.
    wait_time_ = (unsigned int)t2c_get_number("<$WAIT_TIME$>", 
        (long)wait_time_, test_name_, t2c_print_func);
    single_process_ = (int)t2c_get_number("<$SINGLE_PROCESS$>", 
        (long)single_process_, test_name_, t2c_print_func);
    
    // load the requirements
    char* err_ = NULL;
    rcat_ = t2c_rcat_create(rcat_names_, rcat_dir_, &err_);
    if (rcat_ == NULL)
    {
        if (err_)
        {
            TRACE("%s: %s", test_name_, err_);
            free(err_);
            err_ = NULL;
        }
        fprintf(stderr, "\n%s: failed to load catalogue of requirements\n",
            test_name_);
        
        INIT_FAILED("<$T2C_FILE_NAME$>: failed to load catalogue of requirements");
    }

    // prepare absolute path to the suite root
    if (!init_failed_)
    {
        errno = 0;
        if (realpath(suite_root_dir_rel_, suite_root_dir_) == NULL)
        {
            INIT_FAILED(strerror(errno));
        }
    }

    // perform user-defined startup instructions (if any)
    if (!init_failed_)
    {
    <$startup$>
    }
    // end of user-defined startup instructions    
    
    if (init_failed_)
    {
        assert(reason_to_cancel_ != NULL);
        init_fail_reason_ = (char*)strdup(reason_to_cancel_);
        if (init_fail_reason_ == NULL)
        {
            // Out of memory. Do some cleaning-up and exit. 
            // Resource leaks are likely in this case: user-defined
            // cleanup instructions will not be executed.
            if (rcat_ != NULL)
            {
                t2c_rcat_destroy(rcat_);
            }
            fprintf(stderr, "\nOut of memory\n");
            exit(EXIT_FAILURE);
        }
        for (int cur_ = 1; cur_ <= num_tests_; ++cur_)
        {
            tet_delete(cur_, init_fail_reason_);
        }
        fprintf(stderr, 
            "\n%s: initialization failed, see the journal for details\n", 
            test_name_);
    }
    else
    {
        fprintf(stderr, "\nExecuting tests for %s\n", test_name_);
    }
    return;
}

static void 
cleanup_func()
{
    if (!init_failed_)
    {
        fprintf(stderr, "\nPassed %d of %d\n", tests_passed_, num_tests_);
    }
    
    // perform user-defined cleanup instructions (if any)
    <$cleanup$>
    // end of user-defined cleanup instructions

    if (rcat_ != NULL)
    {
        t2c_rcat_destroy(rcat_);
    }
    free(init_fail_reason_);
    return;
}

///////////////////////////////////////////////////////////////////////////
// Tests.
// The name of each test function has the following form:
//     test_<block_index>_<test_index>
// <block_index> is a 1-based index of the <BLOCK> section in the .t2c-file
//     the test was generated from;
// <test_index> is a 1-based index of the test in that <BLOCK> section.
///////////////////////////////////////////////////////////////////////////

<$test_case : join(\n)$>
    
///////////////////////////////////////////////////////////////////////////
// Specials
///////////////////////////////////////////////////////////////////////////

// The list of tests visible for TET
struct tet_testlist tet_testlist[] = {
    <$tet_testlist_item : join(,\n    )$>,    
    {NULL, 0}
};

// The real list of tests in this file. 
FT2CTestFunc test_func[] = {
    <$test_func_item : join(,\n    )$>,    
    NULL
};

// Test launcher.
static void 
test_launcher()
{
    ET2CTestResult tres = T2C_TR_ERROR;
    char* err = NULL;
    
    int tp_ind = tet_thistest - 1;
    int result = t2c_spawn(
        // a test to launch
        test_func[tp_ind], 
        // special startup function to be called once per test
        special_startup_func,   
        // no special cleanup
        NULL,
        // wait time (seconds)
        wait_time_,
        // if nonzero, the same process is used for all the tests
        single_process_,
        // result code of the test
        &tres,
        // error message
        &err);                  
    
    // if the result is T2C_TR_ERROR, there is an error in the template or 
    // API library: this result should have been already handled
    assert(tres != T2C_TR_ERROR);
        
    if (result == 0 && err != NULL)
    {
        t2c_print_error(test_name_, err);
        free(err);
    }
    if (tres == T2C_TR_PASSED)
    {
        ++tests_passed_;
    }
    
    // set TET-specific result code
    tet_result(tet_code_for_t2c_code(tres));
    
    // output a marker
    fprintf(stderr, ".");
    return;
}

static int
special_startup_func()
{
    // This is to distinguish the output from the test if it is executed
    // in a separate process.
    if (!single_process_)
    {
        tet_setcontext();
    }
    return 1;
}

// Tell TET to use startup and cleanup functions we provide.
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;
